@@ -14,13 +14,12 @@ import logging
 
 from pypck import lcn_defs
 from pypck.lcn_addr import LcnAddr
-from pypck.pck_commands import PckGenerator, PckParser
-from pypck.timeout_retry import DEFAULT_TIMEOUT_MSEC
+from pypck.pck_commands import PckParser
 
 _LOGGER = logging.getLogger(__name__)
 
 
-class Input():
+class Input:
     """Parent class for all input data read from LCN-PCHK.
 
     An implementation of :class:`~pypck.input.Input` has to provide easy
@@ -51,16 +50,6 @@ class Input():
         """
         raise NotImplementedError
 
-    def process(self, conn):
-        """Process the :class:`~pypck.input.Input` instance.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        raise NotImplementedError
-
 
 class ModInput(Input):
     """Parent class of all inputs having an LCN module as its source.
@@ -82,17 +71,6 @@ class ModInput(Input):
         """
         return self.logical_source_addr
 
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        if conn.is_ready():  # Skip if we don't have all necessary bus info yet
-            self.logical_source_addr = conn.physical_to_logical(
-                self.physical_source_addr)
 
 # ## Plain text inputs
 
@@ -115,16 +93,6 @@ class AuthUsername(Input):
         if data == PckParser.AUTH_USERNAME:
             return [AuthUsername()]
 
-    def process(self, conn):
-        """Process the :class:`~pypck.input.Input` instance.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        conn.send_command(conn.username)
-
 
 class AuthPassword(Input):
     """Authentication password message received from PCHK."""
@@ -144,16 +112,6 @@ class AuthPassword(Input):
         if data == PckParser.AUTH_PASSWORD:
             return [AuthPassword()]
 
-    def process(self, conn):
-        """Process the :class:`~pypck.input.Input` instance.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        conn.send_command(conn.password)
-
 
 class AuthOk(Input):
     """Authentication ok message received from PCHK."""
@@ -173,15 +131,24 @@ class AuthOk(Input):
         if data == PckParser.AUTH_OK:
             return [AuthOk()]
 
-    def process(self, conn):
-        """Process the :class:`~pypck.input.Input` instance.
 
-        Trigger further actions.
+class AuthFailed(Input):
+    """Authentication failed message received from PCHK."""
 
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
+    @staticmethod
+    def try_parse(data):
+        """Try to parse the given input text.
+
+        Will return a list of parsed Inputs. The list might be empty (but not
+        null).
+
+        :param    data    str:    The input data received from LCN-PCHK
+
+        :return:            The parsed Inputs (never null)
+        :rtype:             List with instances of :class:`~pypck.input.Input`
         """
-        conn.on_auth_ok()
+        if data == PckParser.AUTH_FAILED:
+            return [AuthFailed()]
 
 
 class LcnConnState(Input):
@@ -221,22 +188,28 @@ class LcnConnState(Input):
             inputs = [LcnConnState(False)]
         return inputs
 
-    def process(self, conn):
-        """Process the :class:`~pypck.input.Input` instance.
 
-        Trigger further actions.
+class LicenseError(Input):
+    """LCN bus connected message received from PCHK."""
 
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
+    def __init__(self):
+        """Construct Input object."""
+        super().__init__()
+
+    @staticmethod
+    def try_parse(data):
+        """Try to parse the given input text.
+
+        Will return a list of parsed Inputs. The list might be empty (but not
+        null).
+
+        :param    data    str:    The input data received from LCN-PCHK
+
+        :return:            The parsed Inputs (never null)
+        :rtype:             List with instances of :class:`~pypck.input.Input`
         """
-        if self.is_lcn_connected:
-            _LOGGER.debug('{}: LCN is connected.'.format(conn.connection_id))
-            conn.on_successful_login()
-            conn.send_command(PckGenerator.set_operation_mode(
-                conn.dim_mode, conn.status_mode))
-        else:
-            _LOGGER.debug('{}: LCN is not connected.'.format(
-                conn.connection_id))
+        if data == PckParser.LICENSE_ERROR:
+            return [LicenseError()]
 
 
 class CommandError(Input):
@@ -261,23 +234,8 @@ class CommandError(Input):
         """
         matcher = PckParser.PATTERN_COMMAND_ERROR.match(data)
         if matcher:
-            return [CommandError(matcher.group('message'))]
+            return [CommandError(matcher.group("message"))]
 
-    def process(self, conn):
-        """Process the :class:`~pypck.input.Input` instance.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        _LOGGER.debug('LCN command error: %s', self.message)
-        for address_conn in conn.address_conns.values():
-            if not address_conn.is_group():
-                if address_conn.pck_commands_with_ack:
-                    address_conn.pck_commands_with_ack.popleft()
-                conn.loop.create_task(
-                    address_conn.request_curr_pck_command_with_ack.cancel())
 
 # ## Inputs received from modules
 
@@ -312,30 +270,17 @@ class ModAck(ModInput):
         """
         matcher_pos = PckParser.PATTERN_ACK_POS.match(data)
         if matcher_pos:
-            addr = LcnAddr(int(matcher_pos.group('seg_id')),
-                           int(matcher_pos.group('mod_id')))
+            addr = LcnAddr(
+                int(matcher_pos.group("seg_id")), int(matcher_pos.group("mod_id"))
+            )
             return [ModAck(addr, -1)]
 
         matcher_neg = PckParser.PATTERN_ACK_NEG.match(data)
         if matcher_neg:
-            addr = LcnAddr(int(matcher_neg.group('seg_id')),
-                           int(matcher_neg.group('mod_id')))
-            return [ModAck(addr, matcher_neg.group('code'))]
-
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        # Will replace source segment 0 with the local segment id
-        super().process(conn)
-        if conn.is_ready():
-            module_conn = conn.get_address_conn(self.logical_source_addr)
-            conn.loop.create_task(module_conn.on_ack(self.code,
-                                                     DEFAULT_TIMEOUT_MSEC))
+            addr = LcnAddr(
+                int(matcher_neg.group("seg_id")), int(matcher_neg.group("mod_id"))
+            )
+            return [ModAck(addr, matcher_neg.group("code"))]
 
 
 class ModSk(ModInput):
@@ -368,23 +313,8 @@ class ModSk(ModInput):
         """
         matcher = PckParser.PATTERN_SK_RESPONSE.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
-            return [ModSk(addr, int(matcher.group('id')))]
-
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        if self.physical_source_addr.seg_id == 0:
-            conn.set_local_seg_id(self.reported_seg_id)
-        # Will replace source segment 0 with the local segment id
-        super().process(conn)
-        conn.loop.create_task(conn.status_segment_scan.cancel())
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            return [ModSk(addr, int(matcher.group("id")))]
 
 
 class ModSn(ModInput):
@@ -412,29 +342,43 @@ class ModSn(ModInput):
         """
         matcher = PckParser.PATTERN_SN.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
-            serial = int(matcher.group('sn'), 16)
-            manu = int(matcher.group('manu'), 16)
-            sw_age = int(matcher.group('sw_age'), 16)
-            hw_type = int(matcher.group('hw_type'), 16)
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            serial = int(matcher.group("sn"), 16)
+            manu = int(matcher.group("manu"), 16)
+            sw_age = int(matcher.group("sw_age"), 16)
+            hw_type = int(matcher.group("hw_type"), 16)
             return [ModSn(addr, serial, manu, sw_age, hw_type)]
 
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
 
-        Trigger further actions.
+class ModNameComment(ModInput):
+    """Name or comment received from an LCN module."""
 
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
+    def __init__(self, physical_source_addr, command, block_id, text):
+        """Construct ModInput object."""
+        super().__init__(physical_source_addr)
+        self.command = command
+        self.block_id = block_id
+        self.text = text
+
+    @staticmethod
+    def try_parse(data):
+        """Try to parse the given input text.
+
+        Will return a list of parsed Inputs. The list might be empty (but not
+        null).
+
+        :param    data    str:    The input data received from LCN-PCHK
+
+        :return:            The parsed Inputs (never null)
+        :rtype:             List with instances of :class:`~pypck.input.Input`
         """
-        # Will replace source segment 0 with the local segment id
-        super().process(conn)
-        if conn.is_ready():
-            module_conn = conn.get_address_conn(self.logical_source_addr)
-            module_conn.set_serial(self.serial, self.manu,
-                                   self.sw_age, self.hw_type)
-            conn.loop.create_task(module_conn.request_serial.cancel())
+        matcher = PckParser.PATTERN_NAME_COMMENT.match(data)
+        if matcher:
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            command = matcher.group("command")
+            block_id = int(matcher.group("block_id")) - 1
+            text = matcher.group("text")
+            return [ModNameComment(addr, command, block_id, text)]
 
 
 class ModStatusOutput(ModInput):
@@ -476,31 +420,25 @@ class ModStatusOutput(ModInput):
         """
         matcher = PckParser.PATTERN_STATUS_OUTPUT_PERCENT.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
-            return [ModStatusOutput(addr, int(matcher.group('output_id')) - 1,
-                                    float(matcher.group('percent')))]
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            return [
+                ModStatusOutput(
+                    addr,
+                    int(matcher.group("output_id")) - 1,
+                    float(matcher.group("percent")),
+                )
+            ]
 
         matcher = PckParser.PATTERN_STATUS_OUTPUT_NATIVE.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
-            return [ModStatusOutput(addr, int(matcher.group('output_id')),
-                                    float(matcher.group('value')) / 2.)]
-
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        # Will replace source segment 0 with the local segment id
-        super().process(conn)
-        if conn.is_ready():
-            module_conn = conn.get_address_conn(self.logical_source_addr)
-            module_conn.new_input(self)
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            return [
+                ModStatusOutput(
+                    addr,
+                    int(matcher.group("output_id")),
+                    float(matcher.group("value")) / 2.0,
+                )
+            ]
 
 
 class ModStatusRelays(ModInput):
@@ -536,24 +474,12 @@ class ModStatusRelays(ModInput):
         """
         matcher = PckParser.PATTERN_STATUS_RELAYS.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
-            return [ModStatusRelays(addr, PckParser.get_boolean_value(
-                int(matcher.group('byte_value'))))]
-
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        # Will replace source segment 0 with the local segment id
-        super().process(conn)
-        if conn.is_ready():
-            module_conn = conn.get_address_conn(self.logical_source_addr)
-            module_conn.new_input(self)
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            return [
+                ModStatusRelays(
+                    addr, PckParser.get_boolean_value(int(matcher.group("byte_value")))
+                )
+            ]
 
 
 class ModStatusBinSensors(ModInput):
@@ -588,24 +514,12 @@ class ModStatusBinSensors(ModInput):
         """
         matcher = PckParser.PATTERN_STATUS_BINSENSORS.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
-            return [ModStatusBinSensors(addr, PckParser.get_boolean_value(
-                int(matcher.group('byte_value'))))]
-
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        # Will replace source segment 0 with the local segment id
-        super().process(conn)
-        if conn.is_ready():
-            module_conn = conn.get_address_conn(self.logical_source_addr)
-            module_conn.new_input(self)
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            return [
+                ModStatusBinSensors(
+                    addr, PckParser.get_boolean_value(int(matcher.group("byte_value")))
+                )
+            ]
 
 
 class ModStatusVar(ModInput):
@@ -648,86 +562,53 @@ class ModStatusVar(ModInput):
         """
         matcher = PckParser.PATTERN_STATUS_VAR.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
-            var = lcn_defs.Var.var_id_to_var(int(matcher.group('id')) - 1)
-            value = lcn_defs.VarValue.from_native(int(matcher.group('value')))
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            var = lcn_defs.Var.var_id_to_var(int(matcher.group("id")) - 1)
+            value = lcn_defs.VarValue.from_native(int(matcher.group("value")))
             return [ModStatusVar(addr, var, value)]
 
         matcher = PckParser.PATTERN_STATUS_SETVAR.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
-            var = lcn_defs.Var.set_point_id_to_var(
-                int(matcher.group('id')) - 1)
-            value = lcn_defs.VarValue.from_native(int(matcher.group('value')))
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            var = lcn_defs.Var.set_point_id_to_var(int(matcher.group("id")) - 1)
+            value = lcn_defs.VarValue.from_native(int(matcher.group("value")))
             return [ModStatusVar(addr, var, value)]
 
         matcher = PckParser.PATTERN_STATUS_THRS.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
             var = lcn_defs.Var.thrs_id_to_var(
-                int(matcher.group('register_id')) - 1,
-                int(matcher.group('thrs_id')) - 1)
-            value = lcn_defs.VarValue.from_native(int(matcher.group('value')))
+                int(matcher.group("register_id")) - 1, int(matcher.group("thrs_id")) - 1
+            )
+            value = lcn_defs.VarValue.from_native(int(matcher.group("value")))
             return [ModStatusVar(addr, var, value)]
 
         matcher = PckParser.PATTERN_STATUS_S0INPUT.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
-            var = lcn_defs.Var.s0_id_to_var(int(matcher.group('id')) - 1)
-            value = lcn_defs.VarValue.from_native(int(matcher.group('value')))
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
+            var = lcn_defs.Var.s0_id_to_var(int(matcher.group("id")) - 1)
+            value = lcn_defs.VarValue.from_native(int(matcher.group("value")))
             return [ModStatusVar(addr, var, value)]
 
         matcher = PckParser.PATTERN_VAR_GENERIC.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
             var = lcn_defs.Var.UNKNOWN
-            value = lcn_defs.VarValue.from_native(int(matcher.group('value')))
+            value = lcn_defs.VarValue.from_native(int(matcher.group("value")))
             return [ModStatusVar(addr, var, value)]
 
         matcher = PckParser.PATTERN_THRS5.match(data)
         if matcher:
             ret = []
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
             for thrs_id in range(5):
-                var = lcn_defs.Var.var_id_to_var(int(matcher.group('id')) - 1)
+                var = lcn_defs.Var.var_id_to_var(int(matcher.group("id")) - 1)
                 value = lcn_defs.VarValue.from_native(
-                    int(matcher.group('value{:d}'.format(thrs_id + 1))))
+                    int(matcher.group("value{:d}".format(thrs_id + 1)))
+                )
                 ret.append(ModStatusVar(addr, var, value))
             return ret
 
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        # Will replace source segment 0 with the local segment id
-        super().process(conn)
-        if conn.is_ready():
-            address_conn = conn.get_address_conn(self.logical_source_addr)
-            if self.orig_var == lcn_defs.Var.UNKNOWN:
-                self.var = address_conn.\
-                    get_last_requested_var_without_type_in_response()
-            else:
-                self.var = self.orig_var
-
-            if self.var != lcn_defs.Var.UNKNOWN:
-                if address_conn.\
-                    get_last_requested_var_without_type_in_response() == \
-                        self.var:
-                    address_conn.\
-                        set_last_requested_var_without_type_in_response(
-                            lcn_defs.Var.UNKNOWN)  # Reset
-            address_conn.new_input(self)
-
 
 class ModStatusLedsAndLogicOps(ModInput):
     """Status of LEDs and logic-operations received from an LCN module.
@@ -777,32 +658,17 @@ class ModStatusLedsAndLogicOps(ModInput):
         """
         matcher = PckParser.PATTERN_STATUS_LEDSANDLOGICOPS.match(data)
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
 
-            led_states = matcher.group('led_states').upper()
-            states_leds = [lcn_defs.LedStatus(led_state)
-                           for led_state in led_states]
+            led_states = matcher.group("led_states").upper()
+            states_leds = [lcn_defs.LedStatus(led_state) for led_state in led_states]
 
-            logic_op_states = matcher.group('logic_op_states').upper()
-            states_logic_ops = [lcn_defs.LogicOpStatus(logic_op_state)
-                                for logic_op_state in logic_op_states]
-            return [ModStatusLedsAndLogicOps(addr, states_leds,
-                                             states_logic_ops)]
-
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        # Will replace source segment 0 with the local segment id
-        super().process(conn)
-        if conn.is_ready():
-            address_conn = conn.get_address_conn(self.logical_source_addr)
-            address_conn.new_input(self)
+            logic_op_states = matcher.group("logic_op_states").upper()
+            states_logic_ops = [
+                lcn_defs.LogicOpStatus(logic_op_state)
+                for logic_op_state in logic_op_states
+            ]
+            return [ModStatusLedsAndLogicOps(addr, states_leds, states_logic_ops)]
 
 
 class ModStatusKeyLocks(ModInput):
@@ -842,27 +708,13 @@ class ModStatusKeyLocks(ModInput):
         matcher = PckParser.PATTERN_STATUS_KEYLOCKS.match(data)
         states = []
         if matcher:
-            addr = LcnAddr(int(matcher.group('seg_id')),
-                           int(matcher.group('mod_id')))
+            addr = LcnAddr(int(matcher.group("seg_id")), int(matcher.group("mod_id")))
             for i in range(4):
-                state = matcher.group('table{:d}'.format(i))
+                state = matcher.group("table{:d}".format(i))
                 if state is not None:
                     states.append(PckParser.get_boolean_value(int(state)))
             return [ModStatusKeyLocks(addr, states)]
 
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.ModInput`.
-
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-        # Will replace source segment 0 with the local segment id
-        super().process(conn)
-        if conn.is_ready():
-            module_conn = conn.get_address_conn(self.logical_source_addr)
-            module_conn.new_input(self)
 
 # ## Other inputs
 
@@ -898,34 +750,30 @@ class Unknown(Input):
         """
         return self._data
 
-    def process(self, conn):
-        """Process instance of of :class:`~pypck.input.Input`.
 
-        Trigger further actions.
-
-        :param ~pypck.connection.PchkConnectionManager conn: Connection
-                                                             manager object
-        """
-
-
-class InputParser():
+class InputParser:
     """Parse all input objects for given input data."""
 
-    parsers = [AuthUsername,
-               AuthPassword,
-               AuthOk,
-               LcnConnState,
-               CommandError,
-               ModAck,
-               ModSk,
-               ModSn,
-               ModStatusOutput,
-               ModStatusRelays,
-               ModStatusBinSensors,
-               ModStatusVar,
-               ModStatusLedsAndLogicOps,
-               ModStatusKeyLocks,
-               Unknown]
+    parsers = [
+        AuthUsername,
+        AuthPassword,
+        AuthOk,
+        AuthFailed,
+        LcnConnState,
+        LicenseError,
+        CommandError,
+        ModAck,
+        ModNameComment,
+        ModSk,
+        ModSn,
+        ModStatusOutput,
+        ModStatusRelays,
+        ModStatusBinSensors,
+        ModStatusVar,
+        ModStatusLedsAndLogicOps,
+        ModStatusKeyLocks,
+        Unknown,
+    ]
 
     @staticmethod
     def parse(data):
